import uuid
from datetime import datetime

from freezegun import freeze_time
from pytz import UTC

from django.test import Client, TestCase, override_settings
from django.urls import reverse
from django.utils.timezone import localtime

from core.models import Conference, ConferenceMember, DirectMessage, PlatformUser

TEST_CONF_ID = uuid.uuid4()


@override_settings(SELECTED_CONFERENCE_ID=TEST_CONF_ID)
class MessagesTestCase(TestCase):
    def setUp(self):
        self.conference1 = Conference(id=TEST_CONF_ID, slug='foo', name='Foo Conference', is_public=True)
        self.conference1.save()

        self.human_user1 = PlatformUser(username='bernd', user_type=PlatformUser.Type.HUMAN)
        self.human_user1.save()
        self.human_cm1 = ConferenceMember(conference=self.conference1, user=self.human_user1)
        self.human_cm1.save()

        self.human_user2 = PlatformUser(username='björnd', user_type=PlatformUser.Type.HUMAN)
        self.human_user2.save()
        self.human_cm2 = ConferenceMember(conference=self.conference1, user=self.human_user2)
        self.human_cm2.save()

    def test_DirectMessagesReceived(self):
        c = Client()
        resp = c.get(reverse('api:my-messages-received'))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        resp = c.get(reverse('api:my-messages-received'))
        self.assertEqual(resp.json(), [])

        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user1,
            timestamp=datetime(2020, 5, 4, 3, 2, 1, tzinfo=UTC),
            subject='Test Message2',
            body='Message Body2',
        )
        dm.save()
        resp = c.get(reverse('api:my-messages-received'))
        self.assertEqual(
            resp.json(),
            [
                {
                    'id': str(dm.pk),
                    'sender': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                    'recipient': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                    'timestamp': localtime(dm.timestamp).isoformat(),
                    'in_reply_to': None,
                    'subject': dm.subject,
                    'has_responded': False,
                }
            ],
        )

        dm.has_responded = True
        dm.deleted_by_sender = True
        dm.save()
        resp = c.get(reverse('api:my-messages-received'))
        self.assertEqual(
            resp.json(),
            [
                {
                    'id': str(dm.pk),
                    'sender': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                    'recipient': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                    'timestamp': localtime(dm.timestamp).isoformat(),
                    'in_reply_to': None,
                    'subject': dm.subject,
                    'has_responded': True,
                }
            ],
        )

        dm.deleted_by_recipient = True
        dm.save()
        resp = c.get(reverse('api:my-messages-received'))
        self.assertEqual(resp.json(), [])

    def test_DirectMessagesSent(self):
        c = Client()
        resp = c.get(reverse('api:my-messages-sent'))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        resp = c.get(reverse('api:my-messages-sent'))
        self.assertEqual(resp.json(), [])

        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user1,
            recipient=self.human_user2,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        dm.save()
        resp = c.get(reverse('api:my-messages-sent'))
        self.assertEqual(
            resp.json(),
            [
                {
                    'id': str(dm.pk),
                    'sender': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                    'recipient': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                    'timestamp': localtime(dm.timestamp).isoformat(),
                    'in_reply_to': None,
                    'subject': dm.subject,
                }
            ],
        )

        dm.deleted_by_recipient = True
        dm.save()
        resp = c.get(reverse('api:my-messages-sent'))
        self.assertEqual(len(resp.json()), 1)

        dm.deleted_by_sender = True
        dm.save()
        resp = c.get(reverse('api:my-messages-sent'))
        self.assertEqual(resp.json(), [])

    def test_DirectMessageReceived(self):
        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user1,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        dm.save()

        other_dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user2,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        other_dm.save()

        c = Client()
        resp = c.get(reverse('api:my-message-received', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        resp = c.get(reverse('api:my-message-received', kwargs={'pk': str(other_dm.pk)}))
        self.assertEqual(resp.status_code, 404)

        resp = c.get(reverse('api:my-message-received', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(
            resp.json(),
            {
                'id': str(dm.pk),
                'sender': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                'recipient': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                'timestamp': localtime(dm.timestamp).isoformat(),
                'in_reply_to': None,
                'subject': dm.subject,
                'body': dm.body,
                'has_responded': False,
            },
        )

        dm.has_responded = True
        dm.deleted_by_sender = True
        dm.save()
        resp = c.get(reverse('api:my-message-received', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(
            resp.json(),
            {
                'id': str(dm.pk),
                'sender': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                'recipient': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                'timestamp': localtime(dm.timestamp).isoformat(),
                'in_reply_to': None,
                'subject': dm.subject,
                'body': dm.body,
                'has_responded': True,
            },
        )

        dm.deleted_by_recipient = True
        dm.save()
        resp = c.get(reverse('api:my-message-received', kwargs={'pk': str(other_dm.pk)}))
        self.assertEqual(resp.status_code, 404)

    def test_DirectMessageSent(self):
        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user1,
            recipient=self.human_user2,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        dm.save()

        other_dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user2,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        other_dm.save()

        c = Client()
        resp = c.get(reverse('api:my-message-sent', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        resp = c.get(reverse('api:my-message-sent', kwargs={'pk': str(other_dm.pk)}))
        self.assertEqual(resp.status_code, 404)

        resp = c.get(reverse('api:my-message-sent', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(
            resp.json(),
            {
                'id': str(dm.pk),
                'sender': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                'recipient': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                'timestamp': localtime(dm.timestamp).isoformat(),
                'in_reply_to': None,
                'subject': dm.subject,
                'body': dm.body,
            },
        )

        dm.deleted_by_recipient = True
        dm.save()
        resp = c.get(reverse('api:my-message-sent', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(
            resp.json(),
            {
                'id': str(dm.pk),
                'sender': {'username': self.human_user1.username, 'display_name': self.human_user1.display_name},
                'recipient': {'username': self.human_user2.username, 'display_name': self.human_user2.display_name},
                'timestamp': localtime(dm.timestamp).isoformat(),
                'in_reply_to': None,
                'subject': dm.subject,
                'body': dm.body,
            },
        )

        dm.deleted_by_sender = True
        dm.save()
        resp = c.get(reverse('api:my-message-sent', kwargs={'pk': str(other_dm.pk)}))
        self.assertEqual(resp.status_code, 404)

    def test_DirectMessagesSend(self):
        c = Client()
        resp = c.post(reverse('api:my-send-message'))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        with freeze_time(datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC)):
            resp = c.post(
                reverse('api:my-send-message'),
                {
                    'recipient': self.human_user2.username,
                    'subject': 'Message Subject',
                    'body': 'Message Body',
                },
            )
        self.assertEqual(resp.status_code, 201)
        self.assertEqual(DirectMessage.objects.all().count(), 1)
        dm = DirectMessage.objects.get()
        self.assertEqual(dm.conference, self.conference1)
        self.assertEqual(dm.sender, self.human_user1)
        self.assertEqual(dm.recipient, self.human_user2)
        self.assertEqual(dm.timestamp, datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC))
        self.assertEqual(dm.subject, 'Message Subject')
        self.assertEqual(dm.body, 'Message Body')
        self.assertFalse(dm.was_read)
        self.assertFalse(dm.has_responded)
        self.assertFalse(dm.deleted_by_recipient)

        with freeze_time(datetime(2020, 3, 5, 7, 9, 11, tzinfo=UTC)):
            resp = c.post(
                reverse('api:my-send-message'),
                {
                    'recipient': self.human_user2.username,
                    'subject': 'Message Subject2',
                    'body': 'Message Body2',
                },
            )

        self.assertEqual(DirectMessage.objects.all().count(), 2)
        dm2 = DirectMessage.objects.get(pk=resp.json()['id'])
        self.assertEqual(dm2.timestamp, datetime(2020, 3, 5, 7, 9, 11, tzinfo=UTC))

        self.human_user1.shadow_banned = True
        self.human_user1.save()
        with freeze_time(datetime(2020, 3, 5, 7, 9, 11, tzinfo=UTC)):
            resp = c.post(
                reverse('api:my-send-message'),
                {
                    'recipient': self.human_user2.username,
                    'subject': 'Message Subject, Sender is shadow banned',
                    'body': 'Message Body, Sender is shadow banned',
                },
            )

        self.assertEqual(DirectMessage.objects.all().count(), 3)
        dm3 = DirectMessage.objects.get(pk=resp.json()['id'])
        self.assertTrue(dm3.deleted_by_recipient)

    def test_DirectMessageDelete(self):
        c = Client()
        resp = c.delete(reverse('api:my-delete-message', kwargs={'pk': str(uuid.uuid4())}))
        self.assertEqual(resp.status_code, 403)
        c.force_login(self.human_user1)

        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user1,
            recipient=self.human_user2,
            timestamp=datetime(2020, 5, 4, 3, 2, 1, tzinfo=UTC),
            subject='Test Message',
            body='Message Body',
        )
        dm.save()

        resp = c.delete(reverse('api:my-delete-message', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(resp.status_code, 204)
        dm.refresh_from_db()
        self.assertTrue(dm.deleted_by_sender)
        self.assertFalse(dm.deleted_by_recipient)

        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user1,
            timestamp=datetime(2020, 1, 2, 3, 4, 5, tzinfo=UTC),
            subject='Test Message2',
            body='Message Body2',
        )
        dm.save()
        resp = c.delete(reverse('api:my-delete-message', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(resp.status_code, 204)
        dm.refresh_from_db()
        self.assertFalse(dm.deleted_by_sender)
        self.assertTrue(dm.deleted_by_recipient)

        dm = DirectMessage(
            conference=self.conference1,
            sender=self.human_user2,
            recipient=self.human_user1,
            timestamp=datetime(2020, 2, 2, 2, 2, 2, tzinfo=UTC),
            deleted_by_sender=True,
            subject='Test Message3',
            body='Message Body3',
        )
        dm.save()
        resp = c.delete(reverse('api:my-delete-message', kwargs={'pk': str(dm.pk)}))
        self.assertEqual(resp.status_code, 204)
        self.assertEqual(DirectMessage.objects.filter(pk=dm.pk).count(), 0)
