import re
from datetime import timedelta

from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth.views import INTERNAL_RESET_SESSION_TOKEN
from django.core import mail
from django.http import SimpleCookie
from django.test import override_settings
from django.urls import reverse
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import override as override_locale

from core.models import (
    PlatformUser,
)

from backoffice.tests import BackOfficeTestCase


class RegistrationTests(BackOfficeTestCase):
    @override_locale('en')
    @override_settings(RATELIMIT_ENABLE=False)
    def test_registration(self):
        resp = self.client.get(reverse('backoffice:signup'))
        self.assertEqual(len(mail.outbox), 0)

        resp = self.client.post(reverse('backoffice:signup'), {'username': 'unittest', 'password1': 'mis', 'password2': 'match', 'email': 'doesnot@exist.test'})
        self.assertTrue(resp.context_data['form']['password2'].errors)
        self.assertEqual(len(mail.outbox), 0)

        resp = self.client.post(
            reverse('backoffice:signup'),
            {
                'username': 'unittest',
                'password1': 'Fnord4711',
                'password2': 'Fnord4711',
                'email': 'unittest@example.com',
            },
        )
        self.assertRedirects(resp, reverse('backoffice:account_activation_sent'))
        self.assertEqual(len(mail.outbox), 1)
        sent_mail = mail.outbox[0]
        self.assertEqual(sent_mail.to, ['unittest@example.com'])
        self.assertEqual(sent_mail.subject, 'Activate your testserver account')
        reset_url_pattern = reverse('backoffice:signup_activate', kwargs={'uid_b64': 'uidb64', 'channel_id': 42, 'token': 'token-token'})
        reset_url_pattern = re.escape(reset_url_pattern)
        reset_url_pattern = reset_url_pattern.replace('uidb64', '([^/]+)')
        reset_url_pattern = reset_url_pattern.replace('42', r'(\d+)')
        reset_url_pattern = reset_url_pattern.replace('token\\-token', '([^/]+)')
        m = re.search(reset_url_pattern, sent_mail.body)
        self.assertIsNotNone(m, 'confirm url is missing in sent mail!')

        # now click the activation link and check that the user gets activated
        u = PlatformUser.objects.get(username='unittest')
        self.assertFalse(u.is_active)
        resp = self.client.get(m.group(0), follow=True)
        self.assertRedirects(resp, f"{reverse('backoffice:login')}?next={reverse('backoffice:index')}")
        u.refresh_from_db()
        self.assertTrue(u.is_active)
        self.assertListEqual(list(u.conferences.all()), list(self.conf.users.filter(user__username='unittest')))


class PasswordResetTest(BackOfficeTestCase):
    @override_settings(LANGUAGE_CODE='en', AUTH_PASSWORD_VALIDATORS=[])
    def test_password_reset(self):
        resp = self.client.get(reverse('backoffice:password_reset'))
        self.assertNotContains(resp, 'Reset password link invalid')

    @override_settings(LANGUAGE_CODE='en', AUTH_PASSWORD_VALIDATORS=[])
    def test_invalid_password_reset_link(self):
        uidb = urlsafe_base64_encode(force_bytes(self.user.pk))
        expired_token = default_token_generator._make_token_with_timestamp(
            self.user, default_token_generator._num_seconds(default_token_generator._now() - timedelta(days=365)), secret=None
        )
        expired_resp = self.client.get(reverse('backoffice:password_reset_confirm', kwargs={'uidb64': uidb, 'token': expired_token}), follow=True)
        self.assertRedirects(expired_resp, f"{reverse('backoffice:password_reset')}?retry=True")
        self.assertTemplateUsed(expired_resp, 'registration/password_reset.html')

        invalid_session_resp = self.client.get(reverse('backoffice:password_reset_confirm', kwargs={'uidb64': uidb, 'token': 'set-password'}))
        self.assertRedirects(invalid_session_resp, f"{reverse('backoffice:password_reset')}?retry=True")
        self.assertNotIn(INTERNAL_RESET_SESSION_TOKEN, self.client.session)

    @override_settings(LANGUAGE_CODE='en', AUTH_PASSWORD_VALIDATORS=[])
    def test_PasswordResetConfirmView(self):
        self.client.force_login(self.user)
        self.client.cookies = SimpleCookie()
        self.user.set_password('forgotten')
        self.user.save()
        uidb = urlsafe_base64_encode(force_bytes(self.user.pk))
        token = default_token_generator.make_token(self.user)

        resp = self.client.get(reverse('backoffice:password_reset_confirm', kwargs={'uidb64': uidb, 'token': token}))
        self.assertEqual(self.client.session[INTERNAL_RESET_SESSION_TOKEN], token)
        self.assertRedirects(resp, reverse('backoffice:password_reset_confirm', kwargs={'uidb64': uidb, 'token': 'set-password'}))

        resp = self.client.post(
            reverse('backoffice:password_reset_confirm', kwargs={'uidb64': uidb, 'token': 'set-password'}),
            {
                'new_password1': '4',  # chosen by fair dice roll
                'new_password2': '4',  # guranteed to be random
            },
        )
        self.assertRedirects(resp, reverse('backoffice:password_reset_complete'))

        self.user.refresh_from_db()
        self.assertTrue(self.user.check_password('4'))
