from datetime import UTC, datetime, timedelta
from unittest.mock import patch

from django.urls import reverse
from django.utils import timezone

from core.models import (
    ActivityLogEntry,
    Assembly,
    AssemblyMember,
    ConferenceMember,
    Invitation,
    PlatformUser,
)
from core.tests.mock import mocktrans

from backoffice.tests.invitations.mixin import InvitationTestCase


class InvitationHabitatTestCase(InvitationTestCase):
    def setUp(self):
        super().setUp()

        self.assembly = Assembly.objects.create(name='Assembly 1', slug='assembly_1', conference=self.conf)
        self.assembly_manager = PlatformUser.objects.create(username='assembly_manager')
        self.assembly_manager_cm = ConferenceMember.objects.create(conference=self.conf, user=self.assembly_manager)
        self.assembly_manager_am = AssemblyMember.objects.create(
            assembly=self.assembly,
            member=self.assembly_manager,
            is_representative=True,
            can_manage_assembly=True,
        )
        self.habitat = Assembly.objects.create(name='Habitat 1', slug='habitat_1', conference=self.conf, hierarchy=Assembly.Hierarchy.CLUSTER)
        self.habitat_manager = PlatformUser.objects.create(username='habitat_manager')
        self.habitat_manager_cm = ConferenceMember.objects.create(conference=self.conf, user=self.habitat_manager)
        self.habitat_manager_am = AssemblyMember.objects.create(
            assembly=self.habitat,
            member=self.habitat_manager,
            is_representative=True,
            can_manage_assembly=True,
        )


class HabitatInvitationViewsTestCase(InvitationHabitatTestCase):
    def test_send_habitat_assembly_invitation_accept(self):
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        # Accept invitation
        self.assertIsNone(self.assembly.parent)
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.assembly_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.assembly.refresh_from_db()
        self.assertEqual(self.assembly.parent, self.habitat)
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

    def test_send_assembly_habitat_invitation_accept(self):
        _response, invitation = self.send_successful_invitation(
            user=self.assembly_manager,
            requester_id=self.assembly.id,
            requested_id=self.habitat.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        # Accept invitation
        self.assertIsNone(self.assembly.parent)
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.habitat_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.assembly.refresh_from_db()
        self.assertEqual(self.assembly.parent, self.habitat)
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

    def test_send_habitat_invitation_withdraw(self):
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        self.send_action(
            user=self.habitat_manager,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.WITHDRAWN,
        )

        # Test if we can send the invitation again
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
            exprected_count=2,
        )

    def test_send_habitat_invitation_reject(self):
        with patch.object(timezone, 'now', return_value=datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)):
            _response, invitation = self.send_successful_invitation(
                user=self.habitat_manager,
                requester_id=self.habitat.id,
                requested_id=self.assembly.id,
                invitation_type=Invitation.InvitationType.HABITAT,
            )

            self.send_action(
                user=self.assembly_manager,
                invitation=invitation,
                action='reject',
                expected_state=Invitation.RequestsState.REJECTED,
            )

            self.client.force_login(self.habitat_manager)
            # Try to re-Send invitation from habitat to assembly
            with patch('core.models.invitation._', mocktrans):
                response = self.send_invitation(
                    user=self.habitat_manager,
                    requester_id=self.habitat.id,
                    requested_id=self.assembly.id,
                    invitation_type=Invitation.InvitationType.HABITAT,
                    expected_code=200,
                )

            self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')

        # After the rejection has timed out, can send invitation again
        with (
            patch.object(timezone, 'now', return_value=(datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)) + timedelta(hours=4)),
            patch('core.models.invitation.datetime') as mock_datetime,
        ):
            mock_datetime.now.return_value = datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC) + timedelta(hours=4)
            _response, invitation_2 = self.send_successful_invitation(
                user=self.habitat_manager,
                requester_id=self.habitat.id,
                requested_id=self.assembly.id,
                invitation_type=Invitation.InvitationType.HABITAT,
                exprected_count=2,
            )

            # Accept invitation
            self.assertIsNone(self.assembly.parent)
            activity_log_count = ActivityLogEntry.objects.count()

            self.send_action(
                user=self.assembly_manager,
                invitation=invitation_2,
                action='accept',
                expected_state=Invitation.RequestsState.ACCEPTED,
            )

            self.assembly.refresh_from_db()
            self.assertEqual(self.assembly.parent, self.habitat)
            self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

    def test_send_habitat_invitation_update(self):
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        with patch('backoffice.views.invitations.update._', mocktrans):
            response = self.client.get(
                reverse('backoffice:invitation-edit', kwargs={'pk': invitation.id}),
            )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, 'Invitation__introduction__habitat_translated')
        self.assertContains(response, 'Invitation__type__habitat_translated')

        response = self.send_action(
            user=self.habitat_manager,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, 'Updated comment')

    def test_send_habitat_invitation_invalid_action(self):
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        self.send_action(
            user=self.assembly_manager,
            invitation=invitation,
            action='invalid',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=400,
        )

    def test_no_permission(self):
        _response, invitation = self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        # Cannot accept invitation
        self.assertIsNone(self.assembly.parent)

        self.send_action(
            user=self.habitat_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.assembly.refresh_from_db()
        self.assertIsNone(self.assembly.parent)

        # Cannot reject invitation
        self.assertIsNone(self.assembly.parent)

        self.send_action(
            user=self.habitat_manager,
            invitation=invitation,
            action='reject',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.assembly.refresh_from_db()
        self.assertIsNone(self.assembly.parent)

        self.client.force_login(self.assembly_manager)
        # Cannot withdraw invitation
        self.assertIsNone(self.assembly.parent)
        self.send_action(
            user=self.assembly_manager,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.assembly.refresh_from_db()
        self.assertIsNone(self.assembly.parent)

        # Cannot update invitation
        self.assertIsNone(self.assembly.parent)
        self.send_action(
            user=self.assembly_manager,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )

        self.assembly.refresh_from_db()
        self.assertIsNone(self.assembly.parent)
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, '')

    def test_duplicate_invitation(self):
        self.send_successful_invitation(
            user=self.habitat_manager,
            requester_id=self.habitat.id,
            requested_id=self.assembly.id,
            invitation_type=Invitation.InvitationType.HABITAT,
        )

        # Try to send duplicate invitation

        # Send invitation from habitat to assembly
        with patch('core.models.invitation._', mocktrans):
            response = self.send_invitation(
                user=self.habitat_manager,
                requester_id=self.habitat.id,
                requested_id=self.assembly.id,
                invitation_type=Invitation.InvitationType.HABITAT,
                expected_code=200,
            )
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, f'Invitation__error__already_exists {self.habitat.name} {self.assembly.name} {self.habitat_manager.username}_translated')
        self.assertEqual(Invitation.objects.count(), 1)
