from datetime import UTC, datetime, timedelta
from unittest.mock import patch
from uuid import UUID

from django.http import HttpResponse
from django.urls import reverse
from django.utils import timezone
from django.utils.html import escape

from core.models import (
    ActivityLogEntry,
    ConferenceMember,
    Invitation,
    PlatformUser,
    Team,
    TeamMember,
)
from core.tests.mock import mocktrans

from backoffice.tests.invitations.mixin import InvitationTestCase


class TeamToUserInvitationViewsTestCase(InvitationTestCase):
    def setUp(self):
        super().setUp()

        self.team = Team.objects.create(name='team 1', conference=self.conf)
        self.team_manager = PlatformUser.objects.create(username='assembly_manager')
        self.team_manager_cm = ConferenceMember.objects.create(conference=self.conf, user=self.team_manager)
        self.team_manager_am = TeamMember.objects.create(
            team=self.team,
            user=self.team_manager,
            can_manage=True,
        )

        self.invitee = PlatformUser.objects.create(username='invitee', first_name='First', last_name='Last')
        self.invitee_cm = ConferenceMember.objects.create(conference=self.conf, user=self.invitee)

    def send_successful_invitation(
        self,
        user: PlatformUser | None = None,
        requester_id: str | UUID | None = None,
        requested_id: str | UUID | None = None,
        invitation_type: Invitation.InvitationType = Invitation.InvitationType.TEAM_TO_MEMBER,
        **kwargs,
    ) -> tuple[HttpResponse, Invitation]:
        return super().send_successful_invitation(
            user=self.team_manager if user is None else user,
            requester_id=self.team.uuid if requester_id is None else requester_id,
            requested_id=self.invitee.username if requested_id is None else requested_id,
            invitation_type=invitation_type,
            **kwargs,
        )

    def send_invitation(
        self,
        *,
        user: PlatformUser | None = None,
        requester_id: str | UUID | None = None,
        requested_id: str | UUID | None = None,
        invitation_type: Invitation.InvitationType = Invitation.InvitationType.TEAM_TO_MEMBER,
        **kwargs,
    ) -> HttpResponse:
        return super().send_invitation(
            user=self.team_manager if user is None else user,
            requester_id=self.team.uuid if requester_id is None else requester_id,
            requested_id=self.invitee.username if requested_id is None else requested_id,
            invitation_type=invitation_type,
            **kwargs,
        )

    def test_team_to_user_invitation_accept(self):
        _response, invitation = self.send_successful_invitation()

        # Accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.team.refresh_from_db()
        self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_team_to_user_invitation_requested_in_url_accept(self):
        _response, invitation = self.send_successful_invitation(
            url=reverse(
                'backoffice:invitation-send',
                kwargs={
                    'requester_id': self.team.uuid,
                    'requested_id': self.invitee.uuid,
                    'type': Invitation.InvitationType.TEAM_TO_MEMBER.lower(),
                },
            ),
        )

        # Accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.team.refresh_from_db()
        self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_team_to_user_invitation_withdraw(self):
        _response, invitation = self.send_successful_invitation()

        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.WITHDRAWN,
        )

        # Test if we can send the invitation again
        _response, invitation = self.send_successful_invitation(
            exprected_count=2,
        )

    def test_team_to_user_invitation_reject(self):
        with patch.object(timezone, 'now', return_value=datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)):
            _response, invitation = self.send_successful_invitation()

            self.send_action(
                user=self.invitee,
                invitation=invitation,
                action='reject',
                expected_state=Invitation.RequestsState.REJECTED,
            )

            # Try to re-Send invitation from habitat to assembly
            with patch('core.models.invitation._', mocktrans):
                response = self.send_invitation(
                    expected_code=200,
                )

            self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')

        # After the rejection has timed out, can send invitation again
        with (
            patch.object(timezone, 'now', return_value=(datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)) + timedelta(hours=4)),
            patch('core.models.invitation.datetime') as mock_datetime,
        ):
            mock_datetime.now.return_value = datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC) + timedelta(hours=4)
            _response, invitation_2 = self.send_successful_invitation(exprected_count=2)

            # Accept invitation
            self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
            activity_log_count = ActivityLogEntry.objects.count()

            self.send_action(
                user=self.invitee,
                invitation=invitation_2,
                action='accept',
                expected_state=Invitation.RequestsState.ACCEPTED,
            )

            self.team.refresh_from_db()
            self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
            self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_team_to_user_invitation_update(self):
        _response, invitation = self.send_successful_invitation()

        with patch('backoffice.views.invitations.update._', mocktrans):
            response = self.client.get(
                reverse('backoffice:invitation-edit', kwargs={'pk': invitation.id}),
            )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, 'Invitation__introduction__team_translated')
        self.assertContains(response, 'Invitation__type__team_to_member_translated')

        response = self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, 'Updated comment')

    def test_team_to_user_invitation_invalid_action(self):
        _response, invitation = self.send_successful_invitation()

        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='invalid',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=400,
        )

    def test_team_to_user_invitation_no_permission(self):
        _response, invitation = self.send_successful_invitation()

        # Cannot accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        # Cannot reject invitation
        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='reject',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()

        # Cannot withdraw invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()

        # Cannot update invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )

        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, '')

    def test_team_to_user_invitation_duplicate_invitation(self):
        self.send_successful_invitation()

        # Try to send duplicate invitation

        # Send invitation from habitat to assembly
        with patch('core.models.invitation._', mocktrans):
            response = self.send_invitation(
                expected_code=200,
            )
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, escape(f'Invitation__error__already_exists {self.team.name} {self.invitee.name} {self.team_manager.name}_translated'))
        self.assertEqual(Invitation.objects.count(), 1)


class MemberToTeamInvitationViewsTestCase(InvitationTestCase):
    def setUp(self):
        super().setUp()

        self.team = Team.objects.create(name='team 1', conference=self.conf)
        self.team_manager = PlatformUser.objects.create(username='assembly_manager')
        self.team_manager_cm = ConferenceMember.objects.create(conference=self.conf, user=self.team_manager)
        self.team_manager_am = TeamMember.objects.create(
            team=self.team,
            user=self.team_manager,
            can_manage=True,
        )

        self.invitee = PlatformUser.objects.create(username='invitee')
        self.invitee_cm = ConferenceMember.objects.create(conference=self.conf, user=self.invitee)

    def send_successful_invitation(
        self,
        user: PlatformUser | None = None,
        requester_id: str | UUID | None = None,
        requested_id: str | UUID | None = None,
        invitation_type: Invitation.InvitationType = Invitation.InvitationType.MEMBER_TO_TEAM,
        **kwargs,
    ) -> tuple[HttpResponse, Invitation]:
        return super().send_successful_invitation(
            user=self.invitee if user is None else user,
            requester_id=self.invitee.uuid if requester_id is None else requester_id,
            requested_id=self.team.name if requested_id is None else requested_id,
            invitation_type=invitation_type,
            **kwargs,
        )

    def send_invitation(
        self,
        *,
        user: PlatformUser | None = None,
        requester_id: str | UUID | None = None,
        requested_id: str | UUID | None = None,
        invitation_type: Invitation.InvitationType = Invitation.InvitationType.MEMBER_TO_TEAM,
        **kwargs,
    ) -> HttpResponse:
        return super().send_invitation(
            user=self.invitee if user is None else user,
            requester_id=self.invitee.uuid if requester_id is None else requester_id,
            requested_id=self.team.name if requested_id is None else requested_id,
            invitation_type=invitation_type,
            **kwargs,
        )

    def test_member_to_team_invitation_accept(self):
        _response, invitation = self.send_successful_invitation()

        # Accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.team.refresh_from_db()
        self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_team_to_user_invitation_requested_in_url_accept(self):
        _response, invitation = self.send_successful_invitation(
            url=reverse(
                'backoffice:invitation-send',
                kwargs={
                    'requester_id': self.invitee.uuid,
                    'requested_id': self.team.uuid,
                    'type': Invitation.InvitationType.MEMBER_TO_TEAM.lower(),
                },
            ),
        )

        # Accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        activity_log_count = ActivityLogEntry.objects.count()
        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.ACCEPTED,
        )

        self.team.refresh_from_db()
        self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_member_to_team_invitation_withdraw(self):
        _response, invitation = self.send_successful_invitation()

        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.WITHDRAWN,
        )

        # Test if we can send the invitation again
        _response, invitation = self.send_successful_invitation(
            exprected_count=2,
        )

    def test_member_to_team_invitation_reject(self):
        with patch.object(timezone, 'now', return_value=datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)):
            _response, invitation = self.send_successful_invitation()

            self.send_action(
                user=self.team_manager,
                invitation=invitation,
                action='reject',
                expected_state=Invitation.RequestsState.REJECTED,
            )

            # Try to re-Send invitation from habitat to assembly
            with patch('core.models.invitation._', mocktrans):
                response = self.send_invitation(
                    expected_code=200,
                )

            self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')

        # After the rejection has timed out, can send invitation again
        with (
            patch.object(timezone, 'now', return_value=(datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC)) + timedelta(hours=4)),
            patch('core.models.invitation.datetime') as mock_datetime,
        ):
            mock_datetime.now.return_value = datetime(2042, 12, 27, 12, 34, 0, 0, tzinfo=UTC) + timedelta(hours=4)
            _response, invitation_2 = self.send_successful_invitation(
                exprected_count=2,
            )

            # Accept invitation
            self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
            activity_log_count = ActivityLogEntry.objects.count()

            self.send_action(
                user=self.team_manager,
                invitation=invitation_2,
                action='accept',
                expected_state=Invitation.RequestsState.ACCEPTED,
            )

            self.team.refresh_from_db()
            self.assertTrue(self.team.members.filter(user=self.invitee.id).exists())
            self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_member_to_team_invitation_update(self):
        _response, invitation = self.send_successful_invitation()

        with patch('backoffice.views.invitations.update._', mocktrans):
            response = self.client.get(
                reverse('backoffice:invitation-edit', kwargs={'pk': invitation.id}),
            )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, 'Invitation__introduction__team_translated')
        self.assertContains(response, 'Invitation__type__member_to_team_translated')

        response = self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, 'Updated comment')

    def test_member_to_team_invitation_invalid_action(self):
        _response, invitation = self.send_successful_invitation()

        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='invalid',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=400,
        )

    def test_no_permission(self):
        _response, invitation = self.send_successful_invitation()

        # Cannot accept invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='accept',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        # Cannot reject invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        self.send_action(
            user=self.invitee,
            invitation=invitation,
            action='reject',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        # Cannot withdraw invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='withdraw',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
        )
        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())

        # Cannot update invitation
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        self.send_action(
            user=self.team_manager,
            invitation=invitation,
            action='update',
            expected_state=Invitation.RequestsState.REQUESTED,
            expected_code=403,
            data={
                'requester_id': invitation.requester.id,
                'requested_id': invitation.requested.id,
                'comment': 'Updated comment',
            },
        )

        self.team.refresh_from_db()
        self.assertFalse(self.team.members.filter(user=self.invitee.id).exists())
        invitation.refresh_from_db()
        self.assertEqual(invitation.comment, '')

    def test_duplicate_invitation(self):
        _response, invitation = self.send_successful_invitation()

        # Try to send duplicate invitation

        # Send invitation from habitat to assembly
        with patch('core.models.invitation._', mocktrans):
            response = self.send_invitation(
                expected_code=200,
            )
        self.assertTemplateUsed(response, 'backoffice/invitations/create_edit.html')
        self.assertContains(response, f'Invitation__error__already_exists {self.invitee.name} {self.team.name} {self.invitee.name}_translated')
        self.assertEqual(Invitation.objects.count(), 1)
