from django.contrib.messages import ERROR
from django.contrib.messages.storage.base import Message
from django.contrib.messages.test import MessagesTestMixin
from django.urls import reverse
from django.utils.translation import activate
from django.utils.translation import gettext as _

from core.models import ActivityLogEntry, PlatformUser, Team, TeamMember

from backoffice.tests.base import BackOfficeTestCase


class TeamMemberDeleteViewTestCase(BackOfficeTestCase):
    def setUp(self):
        super().setUp()
        self.staff_2 = PlatformUser.objects.create(username='test_staff_2', is_staff=True)
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
            visibility=Team.TeamVisibility.PUBLIC,
        )
        self.team_member_staff = TeamMember.objects.create(team=self.team, user=self.staff, can_manage=True)
        self.team_member_staff_2 = TeamMember.objects.create(team=self.team, user=self.staff_2, can_manage=True)
        self.team_member_user = TeamMember.objects.create(team=self.team, user=self.user)

    def test_remove_member_admin(self):
        activate('en')
        self.client.force_login(self.admin)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
        )
        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
        self.assertTrue(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count)
        self.assertContains(response, _('TeamMember__delete__warning__header'))

        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_remove_member_staff(self):
        activate('en')
        self.client.force_login(self.staff)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
        )
        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
        self.assertTrue(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count)
        self.assertContains(response, _('TeamMember__delete__warning__header'))

        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_leave_member(self):
        activate('en')
        self.client.force_login(self.user)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
        )
        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
        self.assertTrue(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count)
        self.assertContains(response, _('TeamMember__leave__warning__header'))

        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

    def test_cannot_remove_last_manager(self):
        activate('en')
        self.client.force_login(self.admin)
        activity_log_count = ActivityLogEntry.objects.count()

        # Remove the first manager
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_staff_2.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.staff_2).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

        # Remove a normal user
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

        # Try to remove the last manager
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_staff.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
        self.assertTrue(TeamMember.objects.filter(user=self.staff).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)
        self.assertContains(response, _('TeamMember__delete__cannot_delete_last_manager'))

    def test_cannot_leave_last_manager(self):
        activate('en')
        activity_log_count = ActivityLogEntry.objects.count()

        self.client.force_login(self.staff_2)
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_staff_2.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.staff_2).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

        self.client.force_login(self.user)
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertFalse(TeamMember.objects.filter(user=self.user).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

        self.client.force_login(self.staff)
        response = self.client.post(
            reverse('backoffice:team-member-delete', kwargs={'pk': self.team_member_staff.pk, 'team': self.team.uuid}),
            {'confirmation': 'true'},
        )
        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
        self.assertTrue(TeamMember.objects.filter(user=self.staff).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)
        self.assertContains(response, _('TeamMember__delete__cannot_delete_last_manager'))


class TeamMemberUpdateViewTestCase(MessagesTestMixin, BackOfficeTestCase):
    def setUp(self):
        super().setUp()
        self.staff_2 = PlatformUser.objects.create(username='test_staff_2', is_staff=True)
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
        )
        self.team_member_staff = TeamMember.objects.create(team=self.team, user=self.staff, can_manage=True)
        self.team_member_staff_2 = TeamMember.objects.create(team=self.team, user=self.staff_2, can_manage=True)
        self.team_member_user = TeamMember.objects.create(team=self.team, user=self.user)

    def test_update_member_admin(self):
        activate('en')
        self.client.force_login(self.admin)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            data={'can_manage': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.user, can_manage=True).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_staff.pk, 'team': self.team.uuid}),
            data={'can_manage': 'false'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.staff, can_manage=False).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

    def test_update_member_staff(self):
        activate('en')
        self.client.force_login(self.staff)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_user.pk, 'team': self.team.uuid}),
            data={'can_manage': 'true'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.user, can_manage=True).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

        # Test to remove rights from yourself
        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_staff.pk, 'team': self.team.uuid}),
            data={'can_manage': 'false'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.staff, can_manage=False).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 2)

    def test_cannot_remove_last_manager(self):
        activate('en')
        self.client.force_login(self.staff)
        activity_log_count = ActivityLogEntry.objects.count()
        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_staff_2.pk, 'team': self.team.uuid}),
            data={'can_manage': 'false'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.staff_2, can_manage=False).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)

        response = self.client.post(
            reverse('backoffice:team-member-update', kwargs={'pk': self.team_member_staff.pk, 'team': self.team.uuid}),
            data={'can_manage': 'false'},
        )
        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
        self.assertTrue(TeamMember.objects.filter(user=self.staff, can_manage=True).exists())
        self.assertEqual(ActivityLogEntry.objects.count(), activity_log_count + 1)
        self.assertMessages(response, [Message(ERROR, _('TeamMember__clean__cannot_remove_last_manager'))])
