import logging
from typing import Any

from django.contrib import messages
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView, ListView
from django.views.generic.edit import CreateView, DeleteView, UpdateView

from core.models.badges import Badge, UserBadge
from core.models.users import PlatformUser

from backoffice.forms import (
    BadgeAssignForm,
)
from backoffice.views.badges.mixins import BadgeFormMixin
from backoffice.views.mixins import AssemblyMixin

logger = logging.getLogger(__name__)


class BadgeListView(AssemblyMixin, ListView):
    template_name = 'backoffice/assembly_badges.html'
    assembly_url_param = 'assembly'

    def get_queryset(self, *args, **kwargs):
        return Badge.objects.filter(conference=self.conference, issuing_assembly=self.assembly)


class BadgeCreateView(BadgeFormMixin, CreateView):
    def get_form_kwargs(self) -> dict[str, Any]:
        return {'conference': self.conference, 'assembly': self.assembly, **super().get_form_kwargs()}

    def form_valid(self, form):
        response = super().form_valid(form)
        logger.info('Created badge %(badge)s by %(user)s', {'badge': form.instance, 'user': self.request.user})
        messages.success(self.request, _('Badge__create__successful %(badge)s') % {'badge': form.instance.name})
        return response

    def form_invalid(self, form):
        response = super().form_invalid(form)
        messages.success(self.request, _('Badge__create__failed'))
        return response


class BadgeUpdateView(BadgeFormMixin, UpdateView):
    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['assign_form'] = BadgeAssignForm()
        return ctx

    def form_valid(self, form):
        response = super().form_valid(form)
        logger.info('Updated badge %(badge)s by %(user)s', {'badge': form.instance, 'user': self.request.user})
        messages.success(self.request, _('Badge__update__successful %(badge)s') % {'badge': form.instance.name})
        return response

    def form_invalid(self, form):
        response = super().form_invalid(form)
        messages.success(self.request, _('Badge__update__failed %(badge)s') % {'badge': form.instance.name})
        return response

    def get_queryset(self, *args, **kwargs):
        return Badge.objects.filter(conference=self.conference, issuing_assembly=self.assembly)


class BadgeDeleteView(AssemblyMixin, DeleteView):
    assembly_url_param = 'assembly'
    assembly_management = True
    model = Badge

    def get_object(self, *args, **kwargs):
        obj = super().get_object(*args, **kwargs)
        if obj.issuing_assembly != self.assembly:
            raise self.model.DoesNotExist
        return obj

    def delete(self, *args, **kwargs):
        result = super().delete(*args, **kwargs)
        messages.success(self.request, _('removed'))
        logger.info(
            'Badge "%(badge_name)s" for assembly %(assembly)s removed by %(user)s',
            {'badge_name': self.object, 'assembly': self.assembly, 'user': self.request.user},
        )
        return result

    def get_success_url(self, *args, **kwargs):
        return reverse('backoffice:assembly-badges', kwargs={'assembly': self.assembly.id})


class BadgeAwardView(AssemblyMixin, FormView):
    form_class = BadgeAssignForm
    assembly_url_param = 'assembly'
    assembly_management = True

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['assign_form'] = BadgeAssignForm()
        return ctx

    def form_valid(self, form, **kwargs):
        pk = self.kwargs['pk']
        data = form.cleaned_data
        try:
            user = PlatformUser.objects.only('id', 'username').get(is_active=True, username=data['nickname'])
            badge = get_object_or_404(Badge, pk=pk)
            user_badge, created = UserBadge.objects.redeem_badge(user=user, badge=badge, issuer=self.request.user)
            if created:
                messages.success(self.request, _('Badge__awarded-to-user %(badge)s %(user)s') % {'badge': user_badge.badge, 'user': user})
            else:
                messages.info(self.request, _('Badge__already_owned__by_user %(badge)s %(user)s') % {'badge': user_badge.badge, 'user': user})

            return redirect('backoffice:assembly-badge', assembly=self.assembly.pk, pk=self.kwargs['pk'])
        except PlatformUser.DoesNotExist:
            messages.error(self.request, _('404__User Not Found: ') + data['nickname'])
            return redirect('backoffice:assembly-badge', assembly=self.assembly.pk, pk=self.kwargs['pk'])
