import logging

from django.http import HttpResponseRedirect
from django.urls import reverse
from django.views.generic.edit import FormMixin

from core.models.badges import Badge, BadgeToken

from backoffice.forms import (
    BadgeForm,
    BadgeTokenForm,
    BadgeTokenTimeConstraintFormSet,
)
from backoffice.views.mixins import AssemblyMixin

logger = logging.getLogger(__name__)


class BadgeFormMixin(AssemblyMixin, FormMixin):
    form_class = BadgeForm
    template_name = 'backoffice/assembly_badge.html'

    assembly_url_param = 'assembly'
    assembly_management = True

    def get_success_url(self):
        return reverse('backoffice:assembly-badge', kwargs={'assembly': self.assembly.id, 'pk': self.object.id})


class BadgeTokenFormMixin(FormMixin):
    assembly_url_param = 'assembly'
    assembly_management = True
    model = BadgeToken
    form_class = BadgeTokenForm
    template_name = 'backoffice/assembly_badge_token.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['badge'] = Badge.objects.get(
            conference=self.conference,
            issuing_assembly=self.assembly,
            pk=self.kwargs.get('badge'),
        )
        if 'create_token_form' not in context and self.object:
            context['create_token_form'] = context.pop('form')
            context['create_token_formset'] = BadgeTokenTimeConstraintFormSet(instance=self.object)
        elif 'create_token_form' not in context:
            context['create_token_form'] = context.pop('form')
            context['create_token_formset'] = BadgeTokenTimeConstraintFormSet()
        return context

    def get_success_url(self):
        return reverse('backoffice:assembly-badge-redeem-token', kwargs={'assembly': self.assembly.pk, 'badge': self.badge.pk, 'pk': self.object.pk})

    def form_valid(self, form, formset):
        self.object = form.save(commit=False)
        if not hasattr(self.object, 'badge'):
            self.object.badge = self.badge
        self.object.save()
        formset.instance = self.object
        formset.save()
        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form, formset, additional_context=None):
        ctx = self.get_context_data(create_token_form=form, create_token_formset=formset)
        if additional_context:
            ctx.update(additional_context)
        return self.render_to_response(ctx)

    def post(self, request, *args, **kwargs):
        try:
            self.object = self.get_object()
        except AttributeError:
            self.object = None
        self.badge = Badge.objects.get(
            conference=self.conference,
            issuing_assembly=self.assembly,
            pk=kwargs.get('badge'),
        )
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        formset = BadgeTokenTimeConstraintFormSet(self.request.POST, instance=self.object if hasattr(self, 'object') else None)
        if form.is_valid() and formset.is_valid():
            return self.form_valid(form, formset)
        return self.form_invalid(form, formset)
