import json
import logging
from uuid import uuid4

from django.contrib.gis.db import models as gis_models
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _

from core.fields import ConferenceReference
from core.markdown import compile_translated_markdown_fields, store_relationships

logger = logging.getLogger(__name__)


class MapFloor(models.Model):
    class Meta:
        verbose_name = _('MapFloor')
        verbose_name_plural = _('MapFloors')

    id = models.UUIDField(default=uuid4, primary_key=True, editable=False, serialize=False)
    conference = ConferenceReference(related_name='map_floors')

    index = models.SmallIntegerField(help_text=_('MapFloor__index__help'), verbose_name=_('MapFloor__index'))
    name = models.CharField(
        max_length=200,
        unique=True,
        help_text=_('MapFloor__name__help'),
        verbose_name=_('MapFloor__name'),
    )

    @property
    def code_slug(self) -> str:
        # just return the hex version without the hyphens
        return self.id.hex.replace('-', '')

    def __str__(self):
        return f'[{self.index}] {self.name}'


class MapPOI(models.Model):
    class Meta:
        verbose_name = _('MapPOI')
        verbose_name_plural = _('MapPOIs')

    id = models.UUIDField(default=uuid4, primary_key=True, editable=False, serialize=False)
    conference = ConferenceReference(related_name='pois')

    visible = models.BooleanField(default=True, help_text=_('MapPOI__visible__help'), verbose_name=_('MapPOI__visible'))

    name = models.CharField(max_length=200, unique=True, help_text=_('MapPOI__name__help'), verbose_name=_('MapPOI__name'))
    description = models.TextField(blank=True, null=True, help_text=_('MapPOI__description__help'), verbose_name=_('MapPOI__description'))
    description_html = models.TextField(blank=True)

    is_official = models.BooleanField(default=False, help_text=_('MapPOI__is_official__help'), verbose_name=_('MapPOI__is_official'))

    location_floor = models.ForeignKey(
        MapFloor,
        blank=True,
        null=True,
        related_name='pois',
        on_delete=models.PROTECT,
        help_text=_('MapPOI__location_floor__help'),
        verbose_name=_('MapPOI__location_floor'),
    )

    location_point = gis_models.PointField(
        blank=True,
        null=True,
        help_text=_('MapPOI__location_point__help'),
        verbose_name=_('MapPOI__location_point'),
    )

    def __str__(self):
        return self.name

    def clean(self):
        if self.location_point or self.location_floor:
            if not self.location_point or not self.location_floor:
                raise ValidationError(_('MapPOI__need-both-floor-and-point'))

    def save(self, *args, update_fields=None, **kwargs):
        if update_fields is None or 'description' in update_fields:
            render_results = compile_translated_markdown_fields(self, self.conference, 'description')
            store_relationships(self.conference, self, render_results)

        return super().save(*args, update_fields=update_fields, **kwargs)

    def get_location_floor_index(self):
        return self.location_floor.index if self.location_floor else None

    def get_location_point_xy(self):
        """Get the location_point's X/Y coordinate as array of (two) floats, if it's a valid coordinate - return None otherwise."""
        if not self.location_point or not self.location_point.valid:
            return None

        return [self.location_point.x, self.location_point.y]

    def get_location_point_as_json(self):
        return json.dumps(self.get_location_point_xy() or '')
