from unittest.mock import patch

from django.contrib.contenttypes.models import ContentType
from django.test import override_settings
from django.urls import reverse
from django.utils.translation import override as override_locale

from core.markdown import render_markdown
from core.models import (
    BulletinBoardEntry,
    ConferenceMember,
    MarkdownMeta,
    PlatformUser,
)
from core.models.markdown import MetaTypes
from core.tests.mock import mocktrans

from plainui.tests.test_views import ViewsTestBase


class BoardViewTest(ViewsTestBase):
    @override_locale('en')
    def test_BoardView(self):
        other_user = PlatformUser(username='other_user')
        other_user.save()
        cm_ou = ConferenceMember(conference=self.conf, user=other_user, has_ticket=True)
        cm_ou.save()

        e1 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title1', text='some text')
        e1.save()
        e2 = BulletinBoardEntry(conference=self.conf, owner=self.user, is_public=False, title='Title2', text='other text')
        e2.save()
        e3 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title3', text='different text')
        e3.save()

        self.assertNeedsLogin(reverse('plainui:board'))
        resp = self.client.get(reverse('plainui:board'))
        self.assertEqual(resp.context_data['conf'], self.conf)
        # The lists are always in reverse order, because the order_by('-timestamp') in the view
        self.assertListEqual(list(resp.context_data['board']), [e3, e1])

        self.client.force_login(other_user)
        resp = self.client.get(reverse('plainui:board'))
        self.assertEqual(resp.context_data['conf'], self.conf)
        self.assertListEqual(list(resp.context_data['board']), [e3, e1])

        # TODO: test pagination

        self.client.force_login(self.user)
        self.user.shadow_banned = True
        self.user.save()
        e4 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title4', text='some text', hidden=True)
        e4.save()
        e5 = BulletinBoardEntry(conference=self.conf, owner=other_user, title='Title5', text='some text', hidden=True)
        e5.save()
        resp = self.client.get(reverse('plainui:board'))
        self.assertListEqual(list(resp.context_data['board']), [e4, e3, e1])

        self.client.force_login(other_user)
        resp = self.client.get(reverse('plainui:board'))
        self.assertListEqual(list(resp.context_data['board']), [e5])

    @override_locale('en')
    def test_BoardPrivateView(self):
        user2 = PlatformUser(username='testuser2')
        user2.save()
        e1 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title1', text='some text', hidden=True)
        e1.save()
        e2 = BulletinBoardEntry(conference=self.conf, owner=self.user, is_public=False, title='Title2', text='other text')
        e2.save()
        e_other_owner = BulletinBoardEntry(conference=self.conf, owner=user2)
        e_other_owner.save()

        self.assertNeedsLogin(reverse('plainui:board_private'), check_user=True)
        resp = self.client.get(reverse('plainui:board_private'))
        self.assertEqual(resp.context_data['conf'], self.conf)
        self.assertEqual(list(resp.context_data['board']), [e2, e1])

    @override_locale('en')
    @override_settings(RATELIMIT_ENABLE=False)
    @patch('modeltranslation.settings.AVAILABLE_LANGUAGES', ['en', 'de'])
    def test_BoardEntryEditView_get(self):
        e1 = BulletinBoardEntry(conference=self.conf, owner=self.user, is_public=False, title='Title1', text_de='some de text', text_en='some en text')
        e1.save()

        self.assertNeedsLogin(reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}), check_user=True)
        resp = self.client.get(reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}))
        self.assertEqual(resp.context_data['conf'], self.conf)
        self.assertEqual(resp.context_data['form']['is_public'].value(), e1.is_public)
        self.assertEqual(resp.context_data['form']['title'].value(), e1.title)
        self.assertEqual(resp.context_data['form']['text_de'].value(), e1.text_de)
        self.assertEqual(resp.context_data['form']['text_en'].value(), e1.text_en)

        self.assertNeedsLogin(reverse('plainui:board_entry_new'), check_user=True)
        resp = self.client.get(reverse('plainui:board_entry_new'))
        self.assertEqual(resp.context_data['conf'], self.conf)
        self.assertEqual(resp.context_data['form']['is_public'].value(), True)
        self.assertEqual(resp.context_data['form']['title'].value(), None)
        self.assertEqual(resp.context_data['form']['text_de'].value(), None)
        self.assertEqual(resp.context_data['form']['text_en'].value(), None)

    @override_locale('en')
    @override_settings(RATELIMIT_ENABLE=False)
    @patch('modeltranslation.settings.AVAILABLE_LANGUAGES', ['en', 'de'])
    def test_BoardEntryEditView_post(self):
        user2 = PlatformUser(username='testuser2')
        user2.save()
        e1 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title1', text='some text')
        e1.save()
        e2 = BulletinBoardEntry(conference=self.conf, owner=user2)
        e2.save()

        # edit entry with checked is_public
        self.assertNeedsLogin(reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}), post=True, check_user=True)
        resp = self.client.post(
            reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}),
            {
                'title': 'Edit Title',
                'text_de': 'blblblblbl',
                'text_en': 'blblblbl EN',
                'is_public': 'on',
            },
        )
        self.assertRedirects(resp, reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}))
        e1.refresh_from_db()
        self.assertEqual(e1.is_public, True)
        self.assertEqual(e1.text_de, 'blblblblbl')
        self.assertEqual(e1.text_en, 'blblblbl EN')
        self.assertEqual(e1.text_html_de, str(render_markdown(self.conf, 'blblblblbl')))
        self.assertEqual(e1.text_html_en, str(render_markdown(self.conf, 'blblblbl EN')))
        self.assertSetsMessage(resp, 'Bulletin Board Entry updated.')

        # unchecked is_public
        resp = self.client.post(
            reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}),
            {
                'title': 'Edit Title',
                'text_de': 'blblblblbl',
                'text_en': 'blblblblbl ENL',
            },
        )
        self.assertRedirects(resp, reverse('plainui:board_entry_edit', kwargs={'pk': str(e1.pk)}))
        e1.refresh_from_db()
        self.assertEqual(e1.is_public, False)

        # can't edit other users entries
        resp = self.client.post(
            reverse('plainui:board_entry_edit', kwargs={'pk': str(e2.pk)}),
            {
                'title': 'Edit Title',
                'text_de': 'blblblblbl',
                'text_en': 'blblblblbl e',
            },
        )
        self.assertEqual(resp.status_code, 404)

        # create new entry
        self.assertNeedsLogin(reverse('plainui:board_entry_new'), post=True, check_user=True)
        resp = self.client.post(
            reverse('plainui:board_entry_new'),
            {
                'title': 'New Entry',
                'text_de': 'some texty text de [[start]]',
                'text_en': 'some texty text',
            },
        )
        new_entry = BulletinBoardEntry.objects.exclude(pk__in=[e1.pk, e2.pk]).get()
        self.assertRedirects(resp, reverse('plainui:board_entry_edit', kwargs={'pk': str(new_entry.pk)}))
        self.assertEqual(new_entry.conference, self.conf)
        self.assertEqual(new_entry.owner, self.user)
        self.assertEqual(new_entry.is_public, False)
        self.assertEqual(new_entry.hidden, False)
        self.assertEqual(new_entry.title, 'New Entry')
        self.assertEqual(new_entry.text_de, 'some texty text de [[start]]')
        self.assertEqual(new_entry.text_en, 'some texty text')
        self.assertSetsMessage(resp, 'Bulletin Board Entry created.')
        self.assertEqual(
            MarkdownMeta.objects.filter(
                src_type=ContentType.objects.get_for_model(BulletinBoardEntry), src_object_id=new_entry.pk, dst_type=MetaTypes.PAGE_SLUG, dst_object='start'
            ).count(),
            1,
        )

        BulletinBoardEntry.objects.all().delete()
        self.user.shadow_banned = True
        self.user.save()
        resp = self.client.post(
            reverse('plainui:board_entry_new'),
            {
                'title': 'New Entry',
                'text_de': 'some texty text ed',
                'text_en': 'some texty text ne',
            },
        )
        new_entry = BulletinBoardEntry.objects.get()
        self.assertRedirects(resp, reverse('plainui:board_entry_edit', kwargs={'pk': str(new_entry.pk)}))
        self.assertEqual(new_entry.hidden, True)

        # testing the rate limiting: Will block after 3 allowed requests
        self.user.shadow_banned = False
        self.user.save()
        with override_settings(RATELIMIT_ENABLE=True):
            for i in range(3):
                resp = self.client.post(
                    reverse('plainui:board_entry_new'),
                    {
                        'title': 'New Entry' + str(i),
                        'text': 'some texty text',
                    },
                )
                new_entry = BulletinBoardEntry.objects.get(title='New Entry' + str(i))
                self.assertRedirects(resp, reverse('plainui:board_entry_edit', kwargs={'pk': str(new_entry.pk)}))

            with patch('plainui.forms._', mocktrans):
                resp = self.client.post(
                    reverse('plainui:board_entry_new'),
                    {
                        'title': 'New Entry-Ratelimited',
                        'text': 'some texty text-Ratelimited',
                    },
                )
            self.assertContains(resp, 'rate-limited_translated')

    @override_settings(LANGUAGE_CODE='en')
    @override_locale('en')
    def test_BoardEntryDeleteView(self):
        user2 = PlatformUser(username='testuser2')
        user2.save()
        e1 = BulletinBoardEntry(conference=self.conf, owner=self.user, title='Title1', text='some text')
        e1.save()
        e2 = BulletinBoardEntry(conference=self.conf, owner=user2)
        e2.save()

        # delete existing entry
        self.assertNeedsLogin(reverse('plainui:board_entry_delete', kwargs={'pk': str(e1.pk)}), post=True, check_user=True)
        resp = self.client.post(
            reverse('plainui:board_entry_delete', kwargs={'pk': str(e1.pk)}),
        )
        self.assertRedirects(resp, reverse('plainui:board_private'))
        self.assertFalse(BulletinBoardEntry.objects.filter(pk=e1.pk).exists())
        self.assertSetsMessage(resp, 'Bulletin Board Entry deleted.')

        # doesn't delete entry of other user
        resp = self.client.post(
            reverse('plainui:board_entry_delete', kwargs={'pk': str(e2.pk)}),
        )
        self.assertEqual(resp.status_code, 404)
        self.assertTrue(BulletinBoardEntry.objects.filter(pk=e2.pk).exists())
